@@ -92,7 +92,7 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
     }
 
     while (j < n && i < a->nbits ) {
-        j += GETBIT(a, i);
+        j += getbit(a, i);
         i++;
     }
     if (j < n)
@@ -151,7 +151,7 @@ find_last(bitarrayobject *a, int vi)
 
     /* search within top byte */
     for (i = a->nbits - 1; i >= BITS(a->nbits / 8); i--)
-        if (GETBIT(a, i) == vi)
+        if (getbit(a, i) == vi)
             return i;
 
     if (i < 0)  /* not found within top byte */
@@ -172,7 +172,7 @@ find_last(bitarrayobject *a, int vi)
 
     /* search within byte found */
     for (i = BITS(j + 1) - 1; i >= BITS(j); i--)
-        if (GETBIT(a, i) == vi)
+        if (getbit(a, i) == vi)
             return i;
 
     return -1;
@@ -353,8 +353,7 @@ serialize(PyObject *module, PyObject *a)
         return PyErr_NoMemory();
 
 #define aa  ((bitarrayobject *) a)
-    *data = (char) (16 * IS_BE(aa) + BITS(nbytes) - aa->nbits);
-    setunused(aa);
+    *data = (char) (16 * IS_BE(aa) + setunused(aa));
     memcpy(data + 1, aa->ob_item, (size_t) nbytes);
 #undef aa
     result = PyBytes_FromStringAndSize(data, nbytes + 1);
@@ -558,7 +557,7 @@ ba2base(PyObject *module, PyObject *args)
     for (i = 0; i < strsize; i++) {
         x = 0;
         for (k = 0; k < m; k++)
-            x |= GETBIT(aa, m * i + (le ? k : (m - k - 1))) << k;
+            x |= getbit(aa, m * i + (le ? k : (m - k - 1))) << k;
         str[i] = alphabet[x];
     }
     result = Py_BuildValue("s#", str, strsize);
@@ -617,12 +616,16 @@ base2ba(PyObject *module, PyObject *args)
     Py_RETURN_NONE;
 }
 
+/* ------------------- variable length bitarray format ----------------- */
 
-#define PADBITS  3        /* number of padding bits, used in vl_*() */
+/* PADBITS is always 3 - the number of bits that represent the number of
+   padding bits.  The actual number of padding bits is called 'padding'
+   below, and is in range(0, 7). */
+#define PADBITS  3
 
 /* Consume iterator while decoding bytes into bitarray.
    As we don't have access to bitarrays resize() C function, we give this
-   function a bitarray (large enough in most cases).  We manipulate .nbits
+   function a bitarray (large enough in many cases).  We manipulate .nbits
    and .ob_size (using Py_SET_SIZE) directly without having to call resize().
    Whenever we need a larger bitarray, we call .frombytes() with a multiple
    of 7 dummy bytes, such that the added bytes are aligned for the next time
@@ -635,13 +638,14 @@ vl_decode(PyObject *module, PyObject *args)
 {
     const Py_ssize_t ibits = 256;    /* initial number of bits in a */
     PyObject *iter, *item, *res, *a;
-    Py_ssize_t padding = 0;    /* uninitialized warning for some compiles */
+    Py_ssize_t padding = 0;  /* number of pad bits read from header byte */
+    Py_ssize_t i = 0;        /* bit counter */
+    unsigned char b = 0x80;  /* empty stream will raise StopIteration */
     Py_ssize_t k;
-    Py_ssize_t i = 0;          /* bit counter */
-    unsigned char b = 0x80;    /* empty stream will raise StopIteration */
 
-    assert(ibits == 37 * 7 - PADBITS);   /* bits in a 37 byte stream */
-    assert(ibits % 8 == 0);     /* bytes being added are aligned */
+    /* Ensure that bits will be aligned when gowing memory below.
+       Possible values for ibits are: 32, 88, 144, 200, 256, 312, ... */
+    assert((ibits + PADBITS) % 7 == 0 && ibits % 8 == 0);
 
     if (!PyArg_ParseTuple(args, "OO", &iter, &a))
         return NULL;
@@ -651,6 +655,7 @@ vl_decode(PyObject *module, PyObject *args)
     if (ensure_ba_of_length(a, ibits) < 0)
         return NULL;
 
+#define aa  ((bitarrayobject *) a)
     while ((item = PyIter_Next(iter))) {
 #ifdef IS_PY3K
         if (PyLong_Check(item))
@@ -667,20 +672,18 @@ vl_decode(PyObject *module, PyObject *args)
         }
         Py_DECREF(item);
 
-#define aa  ((bitarrayobject *) a)
-        if (i + 6 >= BITS(Py_SIZE(aa))) {
+        assert(i == 0 || (i + PADBITS) % 7 == 0);
+        if (i == aa->nbits) {
             /* grow memory - see above */
-            assert(i % 8 == 0);  /* ensure added dummy bytes are aligned */
-            aa->nbits = i;
-            Py_SET_SIZE(aa, BYTES(i));
-            /* 63 is a multiple 7, such bytes are aligned for next call */
+            assert(i % 8 == 0);  /* added dummy bytes are aligned */
+            /* 63 is a multiple of 7 - bytes will be aligned for next call */
             res = PyObject_CallMethod(a, "frombytes", BYTES_SIZE_FMT,
                                       base64_alphabet, (Py_ssize_t) 63);
             if (res == NULL)
                 return NULL;
             Py_DECREF(res);  /* drop extend result */
         }
-        assert(i + 6 < BITS(Py_SIZE(aa)));
+        assert(i + 6 < aa->nbits);
 
         if (i == 0) {
             padding = (b & 0x70) >> 4;
@@ -697,10 +700,10 @@ vl_decode(PyObject *module, PyObject *args)
         if ((b & 0x80) == 0)
             break;
     }
-    assert(i == 0 || (i + PADBITS) % 7 == 0);
     /* set final length of bitarray */
     aa->nbits = i - padding;
-    Py_SET_SIZE(aa, BYTES(aa->nbits));
+    Py_SET_SIZE(a, BYTES(aa->nbits));
+    assert_nbits(aa);
 #undef aa
 
     if (PyErr_Occurred())       /* from PyIter_Next() */
@@ -720,7 +723,7 @@ vl_encode(PyObject *module, PyObject *a)
     PyObject *result;
     Py_ssize_t padding, n, m, i, k;
     Py_ssize_t j = 0;           /* byte conter */
-    char *data;
+    char *str;
 
     if (ensure_bitarray(a) < 0)
         return NULL;
@@ -731,28 +734,26 @@ vl_encode(PyObject *module, PyObject *a)
     padding = m - aa->nbits;  /* number of pad bits */
     assert(0 <= padding && padding < 7);
 
-    if ((data = (char *) PyMem_Malloc((size_t) n)) == NULL)
+    if ((str = (char *) PyMem_Malloc((size_t) n)) == NULL)
         return PyErr_NoMemory();
 
-    data[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
-    data[0] |= padding << 4;                /* encode number of pad bits */
+    str[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
+    str[0] |= padding << 4;                /* encode number of pad bits */
     for (i = 0; i < 4 && i < aa->nbits; i++)
-        if (GETBIT(aa, i))
-            data[0] |= 0x08 >> i;
+        str[0] |= (0x08 >> i) * getbit(aa, i);
 
     for (i = 4; i < aa->nbits; i++) {
         k = (i - 4) % 7;
         if (k == 0)
-            data[++j] = i + 7 < m ? 0x80 : 0x00;  /* leading bit */
+            str[++j] = i + 7 < m ? 0x80 : 0x00;  /* leading bit */
 
-        if (GETBIT(aa, i))
-            data[j] |= 0x40 >> k;
+        str[j] |= (0x40 >> k) * getbit(aa, i);
     }
 #undef aa
     assert(j + 1 == n);
 
-    result = PyBytes_FromStringAndSize(data, n);
-    PyMem_Free((void *) data);
+    result = PyBytes_FromStringAndSize(str, n);
+    PyMem_Free((void *) str);
     return result;
 }
 
